/*
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER:  You may use this software, and any derivatives created by any person or
 * entity by or on your behalf, exclusively with Microchip?s products.  Microchip and its licensors retain all ownership
 * and intellectual property rights in the accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.  It does not modify Microchip?s standard
 * warranty for its products.  You agree that you are solely responsible for testing the software and determining its
 * suitability.  Microchip has no obligation to modify, test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,
 * BUT NOT LIMITED TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE
 * APPLY TO THIS SOFTWARE, ITS INTERACTION WITH MICROCHIP?S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
 * ANY APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
 * STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, PUNITIVE,
 * EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
 * SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
 * TO THE FULLEST EXTENT ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS.
 *
 *
 * File:   definitions.h
 *
 * General definitions for the project
 *
 */

#ifndef _DEFINITIONS_H_
#define _DEFINITIONS_H_

#include <stdint.h>

//#define USE_BM70             //This definition is in the preprocessor macro of each project configuration
                               //Mode pin is different for the BM70 and BM71 PICtails

// Clock frequency
#define FCY                 16000000                        //8MHz crystal x 4 PLL = 32 MHz (2 clocks per instruction)

//Macros for interrupts and software breakpoints during debugging
#define _ISR_NO_AUTO_PSV    __attribute__((interrupt,no_auto_psv))
#if defined(__DEBUG)
    #define SYS_DEBUG_BreakPoint() __builtin_software_breakpoint()              //or __asm__ volatile (" sdbbp 0")
#else
    #define SYS_DEBUG_BreakPoint()
#endif

//#define BM70_EEPROM_BASE 0x00034000

// Output pins
#define LED_PORT LATA
#define LED_TRIS TRISA
#define LED_D3              LATAbits.LATA0                  //LEDs are active high
#define LED_D4              LATAbits.LATA1
#define LED_D5              LATAbits.LATA2
#define LED_D6              LATAbits.LATA3
#define LED_D7              LATAbits.LATA4
#define LED_D8              LATAbits.LATA5
#define LED_D9              LATAbits.LATA6
#define LED_D10             LATAbits.LATA7

#define BLE_NOT_RESET_PIN   LATGbits.LATG1                  //Active low reset signal to BM70
#define BLE_RX_IND_PIN      LATFbits.LATF7                                      //Low to indicate when micro UART is going to transmit or is transmitting

#ifdef USE_BM70
#define BLE_MODE_PIN        LATBbits.LATB1                                      //Mode pin P2_0, high for app mode, low for test mode
#else
#define BLE_MODE_PIN        LATGbits.LATG2                                      //Mode pin P2_0, high for app mode, low for test mode
#endif

// Input pins
#define SWITCH_S3           PORTDbits.RD6                                       //Switches are active low
#define SWITCH_S4           PORTDbits.RD13
#define SWITCH_S5           PORTAbits.RA7
#define SWITCH_S5_TRIS      TRISAbits.TRISA7
#define SWITCH_S6           PORTDbits.RD7

#ifdef USE_BM70
#define BLE_NOT_CONNECT_PIN PORTGbits.RG3                                       //P0_2 LED0 pin low when connected
#else
#define BLE_NOT_CONNECT_PIN PORTBbits.RB1                                       //P0_2 LED0 pin low when connected
#endif



//UART baud rate
#define BRG_115200          ((FCY + 230400) / 460800 - 1)                       //BRG value for 115,200 baud with BRGH = 1 (with rounding)

//Timer initialization
#define TIMER_OFF           0
#define TIMER_ON_PRESCALE1  0x8000
#define TIMER_ON_PRESCALE8  0x8010
#define TIMER_ON_PRESCALE64 0x8020
#define TIMER_ON_PRESCALE256 0x8030

#define TIMER_1MS_PERIOD    (FCY / 1000 - 1)
#define TIMER_100US_PERIOD  (FCY / 10000 - 1)

//Buffer sizes
#define SIZE_UART_RX_BUFFER 1024
#define SIZE_UART_TX_BUFFER 256
#define SIZE_MSG_IN_BUFFER  5                                                   //Buffer up to five messages from module
#define SIZE_MSG_OUT_BUFFER 3                                                   //Buffer up to three messages to module
#define SIZE_INCOMING_DATA  50                                                  //Characteristic read/write can return 23 bytes, read local specific service returns 45 bytes
#define SIZE_OUTGOING_DATA  24                                                  //Characteristic write can use 23 bytes

//Return values
#define TRUE                1
#define FALSE               0
#define ERROR               (-1)

//Bluetooth service & characteristic handles - These handles are obtained from the BM70 UI Tool

#define PRIVATE_SERVICE_HANDLE          0x8000
#define PRIVATE_CHAR_IN_HANDLE          0x8001
#define PRIVATE_CHAR_IN_VAL_HANDLE      0x8002
#define PRIVATE_CHAR_OUT_HANDLE         0x8003
#define PRIVATE_CHAR_OUT_VAL_HANDLE     0x8004
#define PRIVATE_CHAR_OUT_CONF_HANDLE    0x8005

enum EventOpcode { NoEventOpcode,
                   PasskeyEntryRequest = 0x60,
                   PairingComplete,
                   PasskeyConfirmRequest,
                   AdvertisingReport = 0x70,
                   LeConnectionComplete,
                   DisconnectionComplete,
                   ConnectionParameterUpdateNotify,
                   CommandComplete = 0x80,
                   StatusReport,
                   ConfigureModeStatus = 0x8f,
                   DiscoverAllPrimaryServiceResponse,
                   DiscoverSpecificPrimaryServiceCharacteristicResponse,
                   DiscoverAllCharacteristicDescriptorResponse,
                   CharacteristicValueReceived,
                   ClientWriteCharacteristicValue = 0x98,
                   ReceivedTransparentData = 0x9a
                 };

enum CommandOpcode { NoCommandOpcode,
                     ReadLocalInformation = 0x01,
                     Reset,
                     ReadStatus,
                     ReadADCValue,
                     IntoShutdownMode,
                     DebugCommand,
                     ReadDeviceName,
                     WriteDeviceName,
                     EraseAllPairedDeviceInformation,
                     ReadPairingModeSetting,
                     WritePairingModeSetting,
                     ReadAllPairedDeviceInformation,
                     DeletePairedDevice,
                     DitalIOControl,
                     PWMControl,
                     ReadRSSIValue,
                     WriteAdvertisingData,
                     WriteScanResultData,
                     SetAdvertisingParameter,
                     SetScanParameter = 0x15,
                     SetScanEnable,
                     LeCreateConnection,
                     LeCreateConnectionCancel,
                     ConnectionParameterUpdateRequest,
                     Disconnect = 0x1b,
                     SetAdvertisingEnable,                                          //Previously called invisible setting
                     ReadRemoteDeviceName = 0x1f,
                     DiscoverAllPrimaryServices = 0x30,
                     DiscoverSpecificPrimaryServiceCharacteristics,
                     ReadCharacteristicValue,
                     ReadUsingCharacteristicUUID,
                     WriteCharacteristicValue,
                     EnableTransparent,
                     SendCharacteristicValue = 0x38,
                     UpdateCharacteristicValue,
                     ReadLocalCharacteristicValue,
                     ReadLocalAllPrimaryService,
                     ReadLocalSpecificPrimaryService,
                     SendTransparentData = 0x3f,
                     PasskeyEntryResponse,
                     UserConfirmResponse,
                     PairingRequest,
                     LeaveConfigureMode = 0x52,
                   };
                  
enum StatusResponse { NoStatusResponse,
                      ScanningMode,
                      ConnectingMode,
                      StandbyMode,
                      BroadcastMode = 0x05,
                      TransparentServiceEnabledMode = 0x08,
                      IdleMode,
                      ShutdownMode,
                      ConfigureMode,
                      BleConnectedMode
                    };

enum ErrorCode { CommandSucceeded,
                 UnknownCommand,
                 UnknownConnectionIdentifier,
                 HardwareFailure,
                 AuthenticationFailure = 0x05,
                 PinOrKeyMissing,
                 MemoryCapacityExceeded,
                 ConnectionTimeout,
                 ConnectionLimitExceeded,
                 ACLConnectionAlreadyExists = 0x0b,
                 CommandDisallowed,
                 ConnectionRejectedDueToLimitedResources,
                 ConnectionRejectedDueToSecurityReasons,
                 ConnectionRejectedDueToUnacceptableBD_ADDR,
                 ConnectionAcceptTimeoutExceeded,
                 UnsupportedFeatureOrParameterValue,
                 InvalidCommandParameters,
                 RemoteUserTerminatedConnection,
                 RemoteDeviceTerminatedConnectionDueToLowResources,
                 RemoteDeviceTerminatedConnectionDueToPowerOff,
                 ConnectionTerminatedByLocalHost,
                 PairingNotAllowed = 0x18,
                 UnspecifiedError = 0x1f,
                 InstantPassed = 0x28,
                 PairingWithUnitKeyNotSupported,
                 InsufficientSecurity = 0x2f,
                 ConnectionRejectedDueToNoSuitableChannelFound = 0x39,
                 ControllerBusy,
                 UnacceptableConnectionInterval,
                 DirectedAdvertisingTimeout,
                 ConnectionTerminatedDueToMICFailure,
                 ConnectionFailedToBeEstablished,
                 InvalidHandle = 0x81,
                 ReadNotPermitted,
                 WriteNotPermitted,
                 InvalidPDU,
                 InsufficientAuthentication,
                 RequestNotSupported,
                 InvalidOffset, //Docs say 0x77, but assume typo, actually 0x87
                 InsufficientAuthorization,
                 PrepareQueueFull,
                 AttributeNotFound,
                 AttributeNotLong,
                 InsufficientEncryptionKeySize,
                 InvalidAttributeValueLength,
                 UnlikelyError,
                 InsufficientEncryption,
                 UnsupportedGroutType,
                 InsufficientResources,
                 ApplicationDefinedError = 0xf0,
                 UARTCheckSumError =0xff
               };                    

enum SetAdvertisingEnable { LeaveStandbyMode,
                            EnterStandbyMode,
                            EnterStandbyModeForTrusted,
                            EnterStandbyModeWithBeacon = 0x81,
                            EnterStandbyModeWithBeaconForTrusted
                          };
               
struct MessageIn {
    enum EventOpcode event;                                                     //Event that caused the message
    enum CommandOpcode command;                                                 //Only applicable if the event is 0x80 CommandComplete
    uint16_t length;                                                            //Length of the data packet, not including opcodes and checksum
    uint8_t data[SIZE_INCOMING_DATA];                                           //Data
};
                    
struct MessageOut {
    enum CommandOpcode command;                                                 //Only applicable if the event is 0x80 CommandComplete
    uint16_t length;                                                            //Length of the data packet, not including opcodes and checksum
    uint8_t data[SIZE_OUTGOING_DATA];                                           //Data
};

struct __attribute__ ((packed)) FlashConfigurationLine {                        //Represents line of text file with flash configuration data
   uint16_t address;                                                            //Address is LSB first
   uint8_t length;                                                              //Length value is always 16
   uint8_t data[16];
};
                    

#endif
