/*
 * Copyright (C) 2014 Microchip Technology Inc. and its subsidiaries.  You may use this software and any derivatives
 * exclusively with Microchip products.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS
 * SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE,
 * COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF
 * THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON
 * ALL CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS.
 *
 *
 * File:        print.c
 * Date:        December 27, 2014
 * Compiler:    XC16 v1.23
 *
 * Print messages on LCD display
 *
 */

#include <xc.h>
#include <string.h>
#include "definitions.h"
#include "lcd.h"

//Static variables  only for use in this file
static char deviceNameAndAddress[17] = "";                                      //string to save text for top line of display

//**********************************************************************************************************************
// Text to display on LCD

//const char LCD_BTDemo[] =           " BLE Stopwatch\r\n";
//const char LCD_Version[] =          "Ver ";

//**********************************************************************************************************************
// Print messages on the LCD

void printDeviceNameAndAddress(uint8_t *name, uint8_t *address)
{
    uint8_t i, temp;
    char *textPointer;
    
    LCD_ClearScreen();                                                          //Clear LCD and cursor to start of line 1
    textPointer = deviceNameAndAddress;
    for (i = 0; i < 7; i++) {
        *textPointer++ = *name++;                                               //Copy 7 name bytes
    }
    *textPointer++ = ' ';
    for (i = 2; i < 6; i++) {                                                   //Print last four bytes of MAC address
        temp = address[i] / 16;                                                 //Calculate high byte
        if (temp < 10) {
            *textPointer++ = (char) '0' + temp;                                 //Add high byte number to text string
        }
        else {
            *textPointer++ = (char) 'A' - 10 + temp;                            //Add high byte letter to text string
        }
        temp = address[i] % 16;                                                 //Calculate low byte
        if (temp < 10) {
            *textPointer++ = (char) '0' + temp;                                 //Add low byte number to text string
        }
        else {
            *textPointer++ = (char) 'A' - 10 + temp;                            //Add low byte letter to text string
        }
    }
    *textPointer++ = 0;                                                         //Null terminate
    LCD_WriteString(deviceNameAndAddress);                                      //Show name and address on the top line of the display
}

void printText(char *text)
{
    if (strlen(text) <= 16) {                               //Check that string is not too long
        LCD_ClearScreen();                                  //Clear LCD and cursor to start of line 1
        LCD_WriteString(deviceNameAndAddress);                  //Show name and address on the top line of the display
        LCD_WriteString(text);                              //Show the text string on the bottom line of the display
    }    
}

void printTextAndNumber(char *text, uint16_t number)
{
    int8_t blanking = TRUE;                                 //Blank zeros on left until first non zero digit
    int16_t i, temp;
    
    LCD_ClearScreen();                                      //Clear LCD and cursor to start of line 1
    LCD_WriteString(deviceNameAndAddress);                  //Show name and address on the top line of the display
    LCD_WriteString(text);                                  //Show the text string on the bottom line of the display
    for (i = 10000; i >= 10; i /= 10) {                     //Start with most significant digit and work down
        temp = number / i;                                  //Calculate the digit to show
        number = number % i;                                //Calculate the the remainder for the next digit
        if (blanking == FALSE) {                            //Display the digit if not blanking
            LCD_PutChar((char) '0' + temp);                 //Display digit on LCD
        }
        else if (temp > 0) {                                //Display non zero digit if blanking
            LCD_PutChar((char) '0' + temp);                 //Display digit on LCD
            blanking = FALSE;                               //No longer blanking because printed non zero digit
        }
    }
    LCD_PutChar((char) '0' + number);                       //Display units digit on LCD regardless of blanking
}

//void printModuleVersion(uint8_t *bTVersion)
//{
//    LCD_ClearScreen();                                      //Clear LCD and cursor to start of line 1
//    LCD_WriteString((char *)LCD_BTDemo);                    //Show "BLE Stopwatch" on the top line of the display
//    LCD_WriteString((char *)LCD_Version);                   //Show "Ver " on the bottom line of the display
//    LCD_PutChar((char) '0' + bTVersion[0] / 10);            //Display tens digit on LCD
//    LCD_PutChar((char) '0' + bTVersion[0] % 10);            //Display ones digit on LCD
//    LCD_PutChar((char) '.');
//    LCD_PutChar((char) '0' + bTVersion[1] / 10);            //Display tens digit on LCD
//    LCD_PutChar((char) '0' + bTVersion[1] % 10);            //Display ones digit on LCD
//    LCD_PutChar((char) '.');
//    LCD_PutChar((char) '0' + bTVersion[2] / 10);            //Display tens digit on LCD
//    LCD_PutChar((char) '0' + bTVersion[2] % 10);            //Display ones digit on LCD
//    LCD_PutChar((char) '.');
//    LCD_PutChar((char) '0' + bTVersion[3] / 10);            //Display tens digit on LCD
//    LCD_PutChar((char) '0' + bTVersion[3] % 10);            //Display ones digit on LCD
//}

//void printTextAndTwoNumbers(char *text, uint16_t number, uint16_t number2)
//{
//    int8_t blanking = TRUE;
//    int16_t i, temp;
//    
//    LCD_ClearScreen();                                      //Clear LCD and cursor to start of line 1
//    LCD_WriteString((char *) LCD_BTDemo);                   //Show "BLE Stopwatch" on the top line of the display
//    LCD_WriteString(text);                                  //Show the text sring on the bottom line of the display
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number / i;
//        number = number % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);             //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);             //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number);                     //Display units digit on LCD
//    LCD_PutChar((char) ' ');                            //Display space on LCD
//    blanking = TRUE;
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number2 / i;
//        number2 = number2 % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);             //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);             //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number2);                  //Display units digit on LCD
//}

//void printFourNumbers(uint16_t number1, uint16_t number2, uint16_t number3, uint16_t number4)
//{
//    int8_t blanking = TRUE;
//    int16_t i, temp;
//    
//    LCD_ClearScreen();                                                          //Clear LCD and cursor to start of line 1
//    LCD_WriteString((char *) LCD_BTDemo);                                       //Show "BLE Stopwatch" on the top line of the display
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number1 / i;
//        number1 = number1 % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number1);                                          //Display units digit on LCD
//    LCD_PutChar((char) ' ');                                                    //Display space on LCD
//    blanking = TRUE;
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number2 / i;
//        number2 = number2 % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number2);                                          //Display units digit on LCD
//    LCD_PutChar((char) ' ');                                                    //Display space on LCD
//    blanking = TRUE;
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number3 / i;
//        number3 = number3 % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number3);                                          //Display units digit on LCD
//    LCD_PutChar((char) ' ');                                                    //Display space on LCD
//    blanking = TRUE;
//    for (i = 10000; i >= 10; i /= 10) {
//        temp = number4 / i;
//        number4 = number4 % i;
//        if (blanking == FALSE) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//        }
//        else if (temp > 0) {
//            LCD_PutChar((char) '0' + temp);                                     //Display digit on LCD
//            blanking = FALSE;
//        }
//    }
//    LCD_PutChar((char) '0' + number4);                                          //Display units digit on LCD
//}

void printStopwatch(uint32_t count, uint8_t running)
{
    static char indicator = '*';
    uint32_t temp;

    count = (count + 50) / 100;                                                 //Count is in 0.0001s increments, needs to be 0.01 for 1/100ths
    count = count % 36000000;                                                   //Don't show over 99 hours 

    printText("  ");                                                            //Start stopwatch time two spaces from left 
//    LCD_ClearScreen();                                                          //Clear LCD and cursor to start of line 1
//    LCD_WriteString((char *) LCD_BTDemo);                                       //Show "BLE Stopwatch" on the top line of the display
    temp = count / 3600000;                                                    //Calculate tens of hours 
    count = count % 3600000;                                                    //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display tens of hours digit on LCD
    temp = count / 360000;                                                      //Calculate units of hours 
    count = count % 360000;                                                     //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display units of hours digit on LCD

    LCD_PutChar((char) ':');                                                    //Colon separator
    temp = count / 60000;                                                      //Calculate tens of minutes 
    count = count % 60000;                                                     //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display tens of minutes digit on LCD
    temp = count / 6000;                                                        //Calculate units of minutes 
    count = count % 6000;                                                       //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display units of minutes digit on LCD

    LCD_PutChar((char) ':');                                                    //Colon separator
    temp = count / 1000;                                                        //Calculate tens of seconds 
    count = count % 1000;                                                       //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display tens of seconds digit on LCD
    temp = count / 100;                                                         //Calculate units seconds minutes 
    count = count % 100;                                                        //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display units of seconds digit on LCD

    LCD_PutChar((char) '.');                                                    //Period separator
    temp = count / 10;                                                          //Calculate tenths of seconds 
    count = count % 10;                                                         //Get remainder 
    LCD_PutChar((char) '0' + temp);                                             //Display tenths of seconds digit on LCD
    LCD_PutChar((char) '0' + count);                                            //Display hundredths of seconds digit on LCD

    LCD_PutChar(' ');                                                           //Display blank space
    if (running == TRUE) {
        LCD_PutChar(indicator);                                                 //Display flashing indicator when running
        if (indicator == '*') {
            indicator = ' ';
        }
        else {
            indicator = '*';
        }
    }
    else {
        LCD_PutChar(' ');                                                       //Display blank space to cover previous indicator
    }

}

