/*
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER:  You may use this software, and any derivatives created by any person or
 * entity by or on your behalf, exclusively with Microchip?s products.  Microchip and its licensors retain all ownership
 * and intellectual property rights in the accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.  It does not modify Microchip?s standard
 * warranty for its products.  You agree that you are solely responsible for testing the software and determining its
 * suitability.  Microchip has no obligation to modify, test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,
 * BUT NOT LIMITED TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE
 * APPLY TO THIS SOFTWARE, ITS INTERACTION WITH MICROCHIP?S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
 * ANY APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
 * STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, PUNITIVE,
 * EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
 * SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
 * TO THE FULLEST EXTENT ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS.
 *
 *
 * File:        switches.c
 * Date:        April 20, 2016
 * Compiler:    XC16 v1.26
 *
 * Functions to sample switches
 *
 */

#include <xc.h>
#include "definitions.h"
#include "timers.h"
#include "stdlib.h"

static struct { unsigned S3isPressed : 1; 
                unsigned S6isPressed : 1; 
                unsigned S5isPressed : 1; 
                unsigned S4isPressed : 1; 
                unsigned S3wasPressed : 1; 
                unsigned S6wasPressed : 1; 
                unsigned S5wasPressed : 1; 
                unsigned S4wasPressed : 1; 
              } switchBits = {0, 0, 0, 0, 0, 0, 0, 0};

//static int8_t switchNewVal = FALSE;                                             //Local variable to record that the switches have a new value
static int8_t timerSwitches;                                                    //Timer variable to debounce by sampling periodically
//static uint8_t oldSwitchVal = 0, switchVal = 0;                                 //State of the four switches

//**********************************************************************************************************************
// Initialize the timer for sampling the switches

void switchInitialize(void) {
    timerSwitches = timersGetAndStartTimer(50);                                 //Get a timer to sample the switches every 50ms
}


//**********************************************************************************************************************
// Periodically sample the switches

void switchTasks(void) {
    if (timersIsTimerDone(timerSwitches)) {                                     //Check whether we have waited for debounce
        timersStartTimer(timerSwitches, 20);                                    //Use software timer to set next sample delay
        LED_TRIS |= 0x00ff;                                                     //Set LED pins on PORTA to inputs to sample switches
        if(SWITCH_S3 == 0) {                                                    //See if switch S3 on RD6 is pressed
            if (switchBits.S3isPressed == 0) {                                  //See if it was not pressed the last time it was sampled
                switchBits.S3wasPressed = 1;                                    //New press detected
            }
            switchBits.S3isPressed = 1;                                         //Switch is pressed
        }
        else {
            switchBits.S3isPressed = 0;                                         //Switch is not pressed
        }
        if(SWITCH_S6 == 0) {                                                    //Read switch S6 on RD7 (low is pressed)
            if (switchBits.S6isPressed == 0) {                                  //See if it was not pressed the last time it was sampled
                switchBits.S6wasPressed = 1;                                    //New press detected
            }
            switchBits.S6isPressed = 1;                                         //Switch is pressed
        }
        else {
            switchBits.S6isPressed = 0;                                         //Switch is not pressed
        }
        if(SWITCH_S5 == 0) {                                                    //Read switch S5 on RA7 (low is pressed)
            if (switchBits.S5isPressed == 0) {                                  //See if it was not pressed the last time it was sampled
                switchBits.S5wasPressed = 1;                                    //New press detected
            }
            switchBits.S5isPressed = 1;                                         //Switch is pressed
        }
        else {
            switchBits.S5isPressed = 0;                                         //Switch is not pressed
        }
        LED_TRIS &= 0xff00;                                                     //Set LED pins on PORTA to outputs after sampling the switch on RA7
        if(SWITCH_S4 == 0) {                                                    //Read switch S4 on RD13 (low is pressed)
            if (switchBits.S4isPressed == 0) {                                  //See if it was not pressed the last time it was sampled
                switchBits.S4wasPressed = 1;                                    //New press detected
            }
            switchBits.S4isPressed = 1;                                         //Switch is pressed
        }
        else {
            switchBits.S4isPressed = 0;                                         //Switch is not pressed
        }
    }
}

//**********************************************************************************************************************
// See if switches have changed

int8_t switchS3WasPressed(void) {
    if (switchBits.S3wasPressed == 1) {                                         //Was the switch pressed?
        switchBits.S3wasPressed = 0;                                            //Clear the indicator to wait for the next press
        return TRUE;                                                            //Yes, switch was pressed
    }
    return FALSE;                                                               //No, switch was not pressed
}

int8_t switchS6WasPressed(void) {
    if (switchBits.S6wasPressed == 1) {                                         //Was the switch pressed?
        switchBits.S6wasPressed = 0;                                            //Clear the indicator to wait for the next press
        return TRUE;                                                            //Yes, switch was pressed
    }
    return FALSE;                                                               //No, switch was not pressed
}

int8_t switchS5WasPressed(void) {
    if (switchBits.S5wasPressed == 1) {                                         //Was the switch pressed?
        switchBits.S5wasPressed = 0;                                            //Clear the indicator to wait for the next press
        return TRUE;                                                            //Yes, switch was pressed
    }
    return FALSE;                                                               //No, switch was not pressed
}

int8_t switchS4WasPressed(void) {
    if (switchBits.S4wasPressed == 1) {                                         //Was the switch pressed?
        switchBits.S4wasPressed = 0;                                            //Clear the indicator to wait for the next press
        return TRUE;                                                            //Yes, switch was pressed
    }
    return FALSE;                                                               //No, switch was not pressed
}


